package com.unitesk.requality.core.checkers;

import java.util.ArrayList;

import com.unitesk.requality.core.Attribute;
import com.unitesk.requality.core.TreeNode;

public class NodeAttribChecker implements IChecker {

    public NodeAttribChecker() {
    }

    @Override
    public CheckerProblem[] check(TreeNode node) {
        ArrayList<CheckerProblem> result = new ArrayList<CheckerProblem>();
        for (String key : node.getAttributeKeys()) {
            Attribute attr = node.getAttribute(key);
            String value = attr.getValue().toString();
            switch (attr.getType()) {
            case INT:
                try {
                    Integer.parseInt(value);
                } catch (NumberFormatException e) {
                    result.add(new CheckerProblem(node, "Attribute \"" + key + "\" of type INT has wrong value: \"" + value + "\""));
                }
                break;
            case FLOAT:
                try {
                    Float.parseFloat(value);
                } catch (NumberFormatException e) {
                    result.add(new CheckerProblem(node, "Attribute \"" + key + "\" of type FLOAT has wrong value: \"" + value + "\""));
                }
                break;
            case BOOL:
                if (!"true".equalsIgnoreCase(value) && !"false".equalsIgnoreCase(value)) {
                    result.add(new CheckerProblem(node, "Attribute \"" + key + "\" of type BOOL has wrong value: \"" + value + "\""));
                }
                break;
            default:
                break;
            }
        }
        return result.toArray(new CheckerProblem[0]);
    }

}
