/*
 * Copyright 2009-2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.requality.eclipse.editors.browser;

import java.io.BufferedWriter;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.io.Writer;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.browser.BrowserFunction;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.ide.IDE;
import org.eclipse.ui.navigator.CommonNavigator;
import org.eclipse.ui.operations.UndoRedoActionGroup;
import org.eclipse.ui.part.EditorPart;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.ui.part.IContributedContentsView;
import org.eclipse.ui.texteditor.ITextEditor;
import org.eclipse.ui.views.contentoutline.IContentOutlinePage;
import org.eclipse.ui.views.properties.IPropertySheetPage;
import org.eclipse.ui.views.properties.tabbed.ITabbedPropertySheetPageContributor;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;

import com.unitesk.requality.core.ITreeChangeListener;
import com.unitesk.requality.core.NodeDesc;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.core.transaction.ChangeResource;
import com.unitesk.requality.core.transaction.ITransactionListener;
import com.unitesk.requality.core.transaction.TransactionStorage;
import com.unitesk.requality.core.transaction.TransactionStorage.OperationData;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.RequalityPlugin;
import com.unitesk.requality.eclipse.core.DefaultTreeChangeListener;
import com.unitesk.requality.eclipse.core.ITreeStateListener;
import com.unitesk.requality.eclipse.core.TreesTracker;
import com.unitesk.requality.eclipse.editors.NodeEditorInput;
import com.unitesk.requality.eclipse.ui.cnf.RequalityCNF;
import com.unitesk.requality.nodetypes.Document;
import com.unitesk.requality.nodetypes.Location;
import com.unitesk.requality.nodetypes.Requirement;
import com.unitesk.requality.tools.RequalityCLI;

public class ReqMarker extends EditorPart implements
        ITabbedPropertySheetPageContributor, ISelectionProvider,
        ITransactionListener {
    private static final String DEFAULT_BASE_ENCODING = "UTF-8";
    public static final String ID = "com.unitesk.requality.editors.browser.ReqMarker";

    protected ReqDocBrowser browser;

    private Document doc;
    private String docQId;
    private TreeDB db;
    private Location initialLocation;
    private IProject project;
    private boolean selFlag = true;

    private ReqMarkerOutlinePage reqMarkerOutlinePage;

    // private IUndoContext undoContext;

    public IUndoContext getUndoContext() {
        // use workbench context
        return PlatformUI.getWorkbench().getOperationSupport().getUndoContext();
        // undoContext = new ObjectUndoContext(this.getEditorInput());
        // return undoContext;
    }

    public void openDocument(URL url) {
        String path = url.toString();
        browser.setUrl(path);
    }

    public ReqDocBrowser getBrowser() {
        return browser;
    }

    private ISelectionChangedListener locChangeListener = new ISelectionChangedListener() {

        @Override
        public void selectionChanged(SelectionChangedEvent event) {
            if (browser != null && !browser.isDisposed() && selFlag) {
                IStructuredSelection s = (IStructuredSelection) event
                        .getSelection();
                // TODO: support broken locations (represented as string)
                if (s.getFirstElement() instanceof Location) {
                    Location loc = (Location) s.getFirstElement();
                    browser.setTargetLocation(loc);
                    synchWithExplorer();
                }
            }
            selFlag = true;
        }
    };

    private ITreeChangeListener treeChangeListener = new DefaultTreeChangeListener() {
        public void deleted(NodeDesc node) {
            if (doc.getUUId().equals(node.getUUId())) {
                getSite().getShell().getDisplay().syncExec(new Runnable() {
                    @Override
                    public void run() {
                        getEditorSite().getPage().closeEditor(ReqMarker.this,
                                false);
                    }
                });
            }

            if (node.getParentUUId().equals(doc.getUUId())) {
                if ((TreesTracker.getResourceByNode(doc)).isAccessible()) {
                    if (reqMarkerOutlinePage != null
                            && !reqMarkerOutlinePage.getControl().isDisposed()) {
                        reqMarkerOutlinePage.refresh();
                    }
                }
            }
        }

        @Override
        public void changed(TreeNode node) {
            getSite().getShell().getDisplay().asyncExec(new Runnable() {
                @Override
                public void run() {
                    if (ReqMarker.this.getBrowser().isDisposed())
                        return;
                    if (TreesTracker.getTracker().getProject(db) == null) {
                        getEditorSite().getPage().closeEditor(ReqMarker.this,
                                false);
                        return;
                    }
                    doc.resetCache();
                    browser.updateLocations();
                    if (reqMarkerOutlinePage != null
                            && !reqMarkerOutlinePage.getControl().isDisposed()) {
                        reqMarkerOutlinePage.refresh();
                    }
                }
            });
        }
    };

    private ITreeStateListener dbStateListener = new ITreeStateListener() {

        @Override
        public void open(TreeDB db) {
        }

        @Override
        public void close(TreeDB db) {
            getSite().getShell().getDisplay().asyncExec(new Runnable() {
                @Override
                public void run() {
                    getEditorSite().getPage()
                            .closeEditor(ReqMarker.this, false);
                }
            });
        }

        @Override
        public void delete(IProject proj) {
            if (project.equals(proj)) {
                getSite().getShell().getDisplay().asyncExec(new Runnable() {
                    @Override
                    public void run() {
                        getEditorSite().getPage().closeEditor(ReqMarker.this,
                                false);
                    }
                });
            }
        }
    };

    final class BrowserKeyListener implements KeyListener {
        @Override
        public void keyReleased(KeyEvent e) {
            if ((e.stateMask & SWT.CTRL) != 0) {
                if (e.keyCode == 'r')
                    new AddSelectionToRequirement(ReqMarker.this).run();
                if (e.keyCode == 'd')
                    new RemoveLocationFromRequirement(ReqMarker.this).run();
            }
        }

        @Override
        public void keyPressed(KeyEvent e) {
        }
    }

    // перехватываем более низкоуровневые события, чем
    // IMenuListener.menuAboutToShow.
    // При событии mouseDown запрещаем выполнение
    // IMenuListener.menuAboutToShow,
    // (чтобы событие Right Mouse Down было передано браузеру),
    // но затем при событии mouseUp вызываем IMenuListener.menuAboutToShow
    // искусственно
    // с помощью Menu.setVisible(true)
    // Сделано это, чтобы событие Mouse Click (Right) передавалось в
    // JavaScript браузера.
    // А еще ReqMarker.this.setFocus() делает Page с ReqMarker активным
    final class BrowserMouseListener implements MouseListener {
        @Override
        public void mouseDown(MouseEvent e) {
            if (e.button == 3) {
                allow_popup_menu = false;
            }
            ReqMarker.this.setFocus();
        }

        @Override
        public void mouseUp(MouseEvent e) {
            if (e.button == 1) {
                browser.activateSelectedLocation();
                onSelectedLocationChangeByClick();
            } else if (e.button == 3) {
                allow_popup_menu = true;
                browser.getWebBrowser().getMenu().setVisible(true);
            }
        }

        @Override
        public void mouseDoubleClick(MouseEvent e) {
        }
    }

    public ReqMarker() {
    }

    protected void updateTargetRequirement() {
        List<Location> locationList = getTargetRequirement().getLocations();
        if (locationList.size() == 0)
            return;
        Location location = locationList.get(0);

        setTargetLocation(location);
    }

    public void setTargetLocation(Location loc) {
        if (browser == null || loc == null) {
            return;
        }
        browser.setTargetLocation(loc);
        if (reqMarkerOutlinePage != null
                && !reqMarkerOutlinePage.getControl().isDisposed()) {
            reqMarkerOutlinePage.refresh();
            reqMarkerOutlinePage.setSelection(new StructuredSelection(loc));
        }
    }

    /**
     * <code>saveDocument</code> must be called in transaction
     * 
     * @param jsData
     * 
     * @throws CoreException
     */
    public void saveDocument(final String jsData) throws CoreException {
        IJSCommand saveDocument = new IJSCommand() {
            @Override
            public void onError(Exception e) {
                e.printStackTrace();
            }

            @Override
            public void onComplite(Object result, ReqDocBrowser docBrowser) {
                String htmlContent = (String) result;
                doc.resetCache();
                if (htmlContent != null && htmlContent.length() > 0) {
                    // TODO: transaction (subtransaction)
                    // db.startTransaction();
                    ByteArrayInputStream in;
                    try {
                        in = new ByteArrayInputStream(
                                htmlContent.getBytes(DEFAULT_BASE_ENCODING));
                        doc.setResourceContent(doc.getResourceName(), in,
                                jsData);
                        // db.commit();
                    } catch (Exception e) {
                        // db.rollback();
                        e.printStackTrace();
                    }

                }
            }

            @Override
            public String getJS() {
                return "return serializeDocument();";
            }

            @Override
            public boolean isEvaluate() {
                return true;
            }
        };
        pullBrowser().scheduleJSCommand(saveDocument);
    }

    @Override
    public void doSave(IProgressMonitor monitor) {
    }

    @Override
    public void doSaveAs() {
        throw new UnsupportedOperationException("Unexpected method call");
    }

    @Override
    public void init(IEditorSite site, IEditorInput input)
            throws PartInitException {
        try {
            setSite(site);
            if (input instanceof NodeEditorInput) {
                NodeEditorInput docInput = (NodeEditorInput) input;
                if (!docInput.getStorage().getNode().getType()
                        .equals("Document")
                        && !docInput.getStorage().getNode().getType()
                                .equals("Requirement")) {
                    throw new PartInitException("Wrong input type: "
                            + docInput.getStorage().getNode().getType());
                }
                if (docInput.getStorage().getNode() instanceof Requirement) {
                    Requirement req = (Requirement) docInput.getStorage()
                            .getNode();
                    if (req.getLocations().size() > 0) {
                        doc = (Document) req.getLocations().get(0).getParent();
                        initialLocation = (Location) req.getLocations().get(0);
                    } else {
                        doc = (Document) docInput.getStorage().getNode();
                        initialLocation = (Location) docInput.getData();
                    }
                } else {
                    doc = (Document) docInput.getStorage().getNode();
                    initialLocation = (Location) docInput.getData();
                }
                db = docInput.getStorage().getNode().getTreeDB();
                project = RequalityPlugin.getTreesTracker().getProject(db);
                docQId = doc.getQualifiedId();
            } else if (input instanceof FileEditorInput) {
                MessageDialog
                        .openError(getSite().getShell(),
                                "Unsupported editor open method",
                                "not yet implemented");
            }
        } catch (CoreException e) {
            e.printStackTrace();
        }
        setInput(input);
        setPartName(doc.getId());

        db.addListener(treeChangeListener);
        db.addTransactionListener(this);
        TreesTracker.getTracker().addListener(dbStateListener);
    }

    @Override
    public boolean isDirty() {
        return false;
    }

    @Override
    public boolean isSaveAsAllowed() {
        return false;
    }

    @Override
    public void createPartControl(Composite parent) {
        new UndoRedoActionGroup(getSite(), getUndoContext(), true)
                .fillActionBars(getEditorSite().getActionBars());
        parent.setLayout(new GridLayout(1, false));

        Composite browserPanel = new Composite(parent, SWT.NONE);
        browserPanel.setLayoutData(new GridData(GridData.FILL_BOTH));
        browserPanel.setLayout(new FillLayout());

        browser = new ReqDocBrowser(browserPanel, SWT.NONE);

        browser.getWebBrowser().setMenu(createMenu());

        new BrowserFunction(browser.getWebBrowser(), "setInitialLocation") {
            @Override
            public Object function(Object[] arguments) {
                setTargetLocation(initialLocation);
                return null;
            }
        };

        browser.getWebBrowser().addKeyListener(new BrowserKeyListener());
        browser.getWebBrowser().addMouseListener(new BrowserMouseListener());
        // Get path to file

        // TODO: check for resource existing
        String openPath = doc.getHTMLPath();
        try {
            URL url = new File(openPath).toURI().toURL();
            openDocument(url);
        } catch (MalformedURLException e1) {
            e1.printStackTrace();
        }
        getSite().setSelectionProvider(this);
    }

    private boolean allow_popup_menu;

    private Menu createMenu() {
        MenuManager menuMgr = new MenuManager("#PopupMenu");
        menuMgr.setRemoveAllWhenShown(true);

        menuMgr.addMenuListener(new IMenuListener() {
            // menuAboutToShow вызывается 2 раза:
            // при mouseDown запрещаем создание меню, при mouseUp разрешаем
            public void menuAboutToShow(IMenuManager manager) {
                if (!allow_popup_menu)
                    return;
                manager.add(new AddSelectionToRequirement(ReqMarker.this));

                Requirement req = getTargetRequirement();
                if (req != null && !req.getQualifiedId().equals(
                        Requirement.getTypeRootQId())) {
                    manager.add(new AddSelectionToRequirement(ReqMarker.this, req));
                }
                if (req == null) {
                    req = (Requirement) db.getNode(Requirement.getTypeRootQId());
                }
                manager.add(new AddSelectionToRequirement(ReqMarker.this,
                        req, RequalityCLI.getNextId(req, "Requirement", null)));

                String newName = browser.getSelectionText().trim();
                if (newName.length() == 0 && browser.isRemovable() && !browser.isSelectable()) {
                    Location loc = getSelectedLocation();
                    if (loc != null) {
                        newName = loc.getDescription();
                    }
                }
                if (newName.length() > 0 && newName.length() < 128) {
                    manager.add(new AddSelectionToRequirement(ReqMarker.this,
                            req, RequalityCLI.getNextId(req, "Requirement", null), newName));
                }

                manager.add(new RemoveLocationFromRequirement(ReqMarker.this));

                if (RequalityPlugin.DEBUG) {
                    manager.add(new Action("Show Version") {
                        @Override
                        public void run() {
                            browser.showBrowserInfo();
                        }
                    });
                }

                manager.add(new Action("Export Page...") {
                    @Override
                    public void run() {
                        String htmlContent = browser.serializeDocumentRaw();
                        htmlContent = htmlContent.replace(Activator
                                .getDefault().getJSToolsPath(), "");
                        DirectoryDialog dialog = new DirectoryDialog(getSite()
                                .getShell());
                        String path = dialog.open();
                        if (path != null) {
                            try {
                                copyPageFile(path + File.separator);
                                Writer out = new BufferedWriter(
                                        new OutputStreamWriter(
                                                new FileOutputStream(path
                                                        + File.separator
                                                        + doc.getId()), "UTF8"));
                                out.write(htmlContent);
                                out.close();
                            } catch (IOException e) {
                                e.printStackTrace();
                            }
                        }
                    }

                    private void copyPageFile(String path) throws IOException {
                        String jsPath = Activator.getDefault().getJSToolsPath();
                        new File(path + File.separator + "jquery").mkdirs();
                        copyFile(new File(jsPath + File.separator + "jquery"
                                + File.separator + "jquery.js"), new File(path
                                + File.separator + "jquery" + File.separator
                                + "jquery.js"));
                        copyFile(new File(jsPath + "edit.js"), new File(path
                                + "edit.js"));
                        copyFile(new File(jsPath + "edit.css"), new File(path
                                + "edit.css"));
                    }
                });
                // manager.update(true);
            }
        });
        Menu menu = menuMgr.createContextMenu(browser);
        return menu;
    }

    // TODO: refactor - move to helpers class
    public static void copyFile(File in, File out) throws IOException {
        FileInputStream fis = new FileInputStream(in);
        FileOutputStream fos = new FileOutputStream(out);
        try {
            byte[] buf = new byte[1024];
            int i = 0;
            while ((i = fis.read(buf)) != -1) {
                fos.write(buf, 0, i);
            }
        } catch (IOException e) {
            throw e;
        } finally {
            if (fis != null)
                fis.close();
            if (fos != null)
                fos.close();
        }
    }

    @Override
    public void setFocus() {
        new UndoRedoActionGroup(getSite(), getUndoContext(), true)
                .fillActionBars(getEditorSite().getActionBars());
        browser.setFocus();
    }

    public TreeDB getTreeDB() {
        return db;
    }

    public Document getDocument() {
        return doc;
    }

    /**
     * <code>addSelectionTo</code> must be called in transaction
     * 
     * @param req
     * @param hideLocation
     * @return array of objects. [0] - selection description, [1] - location
     *         node
     * @throws UnsupportedEncodingException
     * @throws CoreException
     */
    public Object[] addSelectionTo(Requirement req, boolean hideLocation)
            throws UnsupportedEncodingException, CoreException {
        String locId = UUID.randomUUID().toString();
        String reqQId = req.getQualifiedId();
        Object[] jsData = browser.createLocation(locId, reqQId);
        String desc = (String) jsData[0];

        // req.getTreeDB().startTransaction();
        saveDocument((String) jsData[1]);
        Location loc = (Location) db.createChildNode(UUID.fromString(locId),
                doc.getUUId(), locId, "Location");
        loc.setDescription(desc);
        if (hideLocation) {
            loc.setHidden(true);
        }
        loc.saveAttributes();
        req.addLocation(doc.getQualifiedId() + "/" + locId);
        req.saveAttributes();
        // req.getTreeDB().commit();
        return new Object[] { (String) jsData[1], loc };
    }

    /**
     * <code>removeLocation</code> must be called in transaction
     * 
     * @param locQId
     * @return
     */
    public String removeLocation(String locQId) {
        Requirement[] reqs = Location.getRequirementsFor(getTreeDB(), locQId);
        // getTreeDB().startTransaction();
        for (Requirement req : reqs) {
            req.removeLocation(locQId);
            req.saveAttributes();
        }
        String locationId = locQId.substring(locQId.lastIndexOf("/") + 1);
        String jsData = browser.deleteLocation(locationId);

        try {
            saveDocument(jsData);
            // TODO: check it
        } catch (Exception e) {
            e.printStackTrace();
        }
        // getTreeDB().commit();
        return jsData;
    }

    @Override
    public void dispose() {
        db.removeListener(treeChangeListener);
        db.removeTransactionListener(this);
        TreesTracker.getTracker().removeListener(dbStateListener);
        getSite().setSelectionProvider(null);
        super.dispose();
    }

    public Requirement getTargetRequirement() {
        Requirement req = (Requirement) Activator.getDefault().getActiveNode(
                "Requirement");
        if (req != null && req.getTreeDB() == doc.getTreeDB()) {
            return req;
        }
        return null;
    }

    public Location getHighlightLocation() {
        String locId = browser.getSelectedLocationId();
        if (locId != null) {
            return doc.getLocation(locId);
        }
        return null;
    }

    public Location getSelectedLocation() {
        Location loc = null;
        String locId = browser.getSelectedLocationId();
        if (locId != null) {
            loc = doc.getLocation(locId);
        }
        if (loc == null) {
            loc = browser.getTargetLocation();
        }
        return loc;
    }

    public void close(boolean save) {
        getSite().getPage().closeEditor(this, save);
    }

    @Override
    public Object getAdapter(@SuppressWarnings("rawtypes") Class required) {
        if (required.equals(this.getClass())) {
            return this;
        }
        if (required.equals(ITextEditor.class)) {
            return null;
        }
        if (IContentOutlinePage.class.equals(required)) {
            if (reqMarkerOutlinePage == null) {
                reqMarkerOutlinePage = new ReqMarkerOutlinePage(this);
                reqMarkerOutlinePage.setInput(doc);
                reqMarkerOutlinePage
                        .addSelectionChangedListener(locChangeListener);
            }
            return reqMarkerOutlinePage;
        }

        if (required == IPropertySheetPage.class) {
            return new TabbedPropertySheetPage(this);
        }
        if (required == IContributedContentsView.class) {
            return new IContributedContentsView() {
                public IWorkbenchPart getContributingPart() {
                    return PlatformUI
                            .getWorkbench()
                            .getActiveWorkbenchWindow()
                            .getActivePage()
                            .findView(
                                    "com.unitesk.requality.views.RequalityExplorer");
                }
            };

        }

        return super.getAdapter(required);
    }

    @Override
    public String getContributorId() {
        return "com.unitesk.requality.views.RequalityPropertyContributor";
    }

    // //////////////////////////////////////////////////////////////////////////
    //
    // Selection provider
    //
    // //////////////////////////////////////////////////////////////////////////
    private List<ISelectionChangedListener> selectionChangeListeners = new ArrayList<ISelectionChangedListener>();

    public void onSelectedLocationChangeByClick() {
        if (reqMarkerOutlinePage != null
                && !reqMarkerOutlinePage.getControl().isDisposed()) {
            selFlag = false;
            String locationId = browser.getSelectedLocationId();
            if (locationId == null || locationId.isEmpty()) {
                fireSelectionChange();
                reqMarkerOutlinePage.setSelection(StructuredSelection.EMPTY);
                return;
            }
            Location location = doc.getLocation(locationId);
            if (location == null) {
                reqMarkerOutlinePage.setSelection(StructuredSelection.EMPTY);
                return;
            }
            reqMarkerOutlinePage
                    .setSelection(new StructuredSelection(location));
            selFlag = true;
            fireSelectionChange();
            synchWithExplorer();
        }
    }

    @Override
    public void addSelectionChangedListener(ISelectionChangedListener listener) {
        if (listener != null) {
            selectionChangeListeners.add(listener);
        }
    }

    @Override
    public void removeSelectionChangedListener(
            ISelectionChangedListener listener) {
        if (listener != null) {
            selectionChangeListeners.remove(listener);
        }
    }

    @Override
    public ISelection getSelection() {
        String locationId = null;

        if (browser.isReady()) {
            locationId = browser.getSelectedLocationId();
            if (locationId != null) {
                Location location = doc.getLocation(locationId);
                if (location != null) {
                    IStructuredSelection selection = new StructuredSelection(
                            location);
                    return selection;
                }
            }
        }
        ISelection selection = PlatformUI.getWorkbench()
                .getActiveWorkbenchWindow().getSelectionService()
                .getSelection(RequalityCNF.ID);
        return selection;
    }

    @Override
    public void setSelection(ISelection selection) {
        // TODO remove
        System.out.println("setSelection: " + selection);
    }

    public void fireSelectionChange() {
        ISelection sel = this.getSelection();
        SelectionChangedEvent event = new SelectionChangedEvent(this, sel);
        for (ISelectionChangedListener listener : selectionChangeListeners) {
            listener.selectionChanged(event);
        }
    }

    public void synchWithExplorer() {
        // synchronization with Requality Explorer
        IViewPart view = PlatformUI.getWorkbench().getActiveWorkbenchWindow()
                .getActivePage().findView(RequalityCNF.ID);
        if (view instanceof CommonNavigator) {
            CommonNavigator navigator = (CommonNavigator) view;
            if (navigator.isLinkingEnabled()) {
                navigator.setLinkingEnabled(true);
            }
        }
    }
    // //////////////////////////////////////////////////////////////////////////
    //
    // Undo/Redo support
    //
    // //////////////////////////////////////////////////////////////////////////
    public boolean redo(final String jsData)
            throws UnsupportedEncodingException, CoreException {
        ReqDocBrowser docBrowser = pullBrowser();
        IJSCommand redo = new IJSCommand() {
            @Override
            public void onError(Exception e) {
                fireSelectionChange();
                e.printStackTrace();
            }

            @Override
            public void onComplite(Object result, ReqDocBrowser docBrowser) {
                fireSelectionChange();
            }

            @Override
            public String getJS() {
                return "return redoAction(" + jsData + ");";
            }

            @Override
            public boolean isEvaluate() {
                return true;
            }
        };
        docBrowser.scheduleJSCommand(redo);
        return true;

    }

    public boolean undo(final String jsData)
            throws UnsupportedEncodingException, CoreException {
        ReqDocBrowser docBrowser = pullBrowser();
        IJSCommand undo = new IJSCommand() {
            @Override
            public void onError(Exception e) {
                fireSelectionChange();
                e.printStackTrace();
            }

            @Override
            public void onComplite(Object result, ReqDocBrowser docBrowser) {
                fireSelectionChange();
            }

            @Override
            public String getJS() {
                return "return undoAction(" + jsData + ");";
            }

            @Override
            public boolean isEvaluate() {
                return true;
            }
        };
        docBrowser.scheduleJSCommand(undo);
        return true;
    }

    private ReqDocBrowser pullBrowser() throws PartInitException {
        ReqDocBrowser docBrowser = getBrowser();
        if (docBrowser.isDisposed()) {
            ReqMarker reqMarker;
            IEditorReference eref = RequalityPlugin.findMarkEditorForDocument(
                    docQId, TreesTracker.getResourceByNode(doc).getProject());
            if (eref != null) {
                reqMarker = (ReqMarker) eref.getEditor(true);
            } else {
                IWorkbenchPage page = PlatformUI.getWorkbench()
                        .getActiveWorkbenchWindow().getActivePage();
                reqMarker = (ReqMarker) IDE.openEditor(page, getEditorInput(),
                        ReqMarker.ID, false);
            }
            reqMarker.getSite().getPage().bringToTop(reqMarker);
            docBrowser = reqMarker.getBrowser();
        }
        docBrowser.setFocus();
        return docBrowser;
    }

    // //////////////////////////////////////////////////////////////////////////
    //
    // Transaction listener
    //
    // //////////////////////////////////////////////////////////////////////////

    @Override
    public void execute(TransactionStorage tStorage) {
    }

    @Override
    public void redo(TransactionStorage tStorage) {
        try {
            for (OperationData opData : tStorage.getLastOperationsData()) {
                if (opData.getOperation() instanceof ChangeResource) {
                    ChangeResource op = (ChangeResource) opData.getOperation();
                    if (isCurrentDocOp(op)) {
                        redo((String) op.getData());
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public void undo(TransactionStorage tStorage) {
        try {
            for (OperationData opData : tStorage.getLastOperationsData()) {
                if (opData.getOperation() instanceof ChangeResource) {
                    ChangeResource op = (ChangeResource) opData.getOperation();
                    if (isCurrentDocOp(op)) {
                        undo((String) op.getData());
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private boolean isCurrentDocOp(ChangeResource op) {
        return op.getNodeUUID().equals(doc.getUUId())
                && doc.getResourceName().endsWith(op.getResourceName())
                && op.getData() != null;
    }
}
