package com.unitesk.requality.eclipse.handlers;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.PlatformUI;

import com.unitesk.requality.core.NodeDesc;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.ui.cnf.RequalityCNF;
import com.unitesk.requality.nodetypes.Location;
import com.unitesk.requality.nodetypes.Requirement;
import com.unitesk.requality.tools.DeepFirstTreeWalker;

public class PasteIntoHandler extends AbstractHandler {

    @SuppressWarnings("serial")
    private class WrongProjectException extends Exception {
    }

    @SuppressWarnings("serial")
    private class PastingIntoLeafException extends Exception {
    }

    @Override
    public Object execute(ExecutionEvent event) throws ExecutionException {
        if (!CopyPasteHelper.getInstance().isEmpty()) {
            try {
                TreeNode parent = RequalityCNF.getSelectedNode();
                TreeNode source = CopyPasteHelper.getInstance().getNode();
                if (!source.getTreeDB().equals(parent.getTreeDB()))
                    throw new WrongProjectException();
                if (source.getType().equals("Requirement")) {
                    for (TreeNode child : parent.getSortedChildren()) {
                        if (child.getType().equals("TestPurpose"))
                            throw new PastingIntoLeafException();
                    }
                }
                NodeDesc nodeDesc = NodeDesc.fromNode(source, true);
                parent.getTreeDB().startTransaction("Copy Tree Node");
                try {
                    TreeNode old = parent.findChild(nodeDesc.getId());
                    if(old != null)
                    {
                    	String oldId = old.getId();
                    	String newId = oldId+"_copy";
                    	int copyCount = 1;
                    	while(parent.findChild(newId+copyCount)!=null)
                    		copyCount++;
                    	nodeDesc.setId(newId+copyCount);
                    }
                	nodeDesc.createNode(parent.getTreeDB(), parent.getUUId(),
                            true);
                    parent.getTreeDB().commit();
                    // reset location names doesn't affect treedb
                    resetLocationNames(parent.getTreeDB().getNode(
                            nodeDesc.getUUId()));
                } catch (IllegalArgumentException e) {
                    parent.getTreeDB().rollback();
                    e.printStackTrace();
                } catch (NullPointerException e) {
                    parent.getTreeDB().rollback();
                    e.printStackTrace();
                } catch (Throwable t) {
                    parent.getTreeDB().rollback();
                    t.printStackTrace();
                } finally {
                    CopyPasteHelper.getInstance().setEmpty();
                }
            } catch (WrongProjectException e) {
                MessageDialog.openError(PlatformUI.getWorkbench()
                        .getActiveWorkbenchWindow().getShell(), "Paste Node",
                        "Can't copy to another project");
            } catch (PastingIntoLeafException e) {
                MessageDialog.openError(PlatformUI.getWorkbench()
                        .getActiveWorkbenchWindow().getShell(), "Paste Node",
                        "Can't paste a node into a leaf node");
            }
        }
        return null;
    }

    private void resetLocationNames(TreeNode root) {
        for (TreeNode node : new DeepFirstTreeWalker(root)) {
            if (node.getType().equals("Requirement")) {
                for (Location l : ((Requirement) node).getLocations()) {
                    l.resetLabel();
                }
            }
        }
    }

    @Override
    public boolean isEnabled() {
        TreeNode target = RequalityCNF.getSelectedNode();
        if (target == null) {
            return false;
        }
        if(CopyPasteHelper.getInstance().isEmpty()) {
            return false;
        }
        TreeNode source = CopyPasteHelper.getInstance().getNode();

        if (!target.getType().equals("Requirement")
                && !target.getType().equals("TestPurpose")) {
            return false;
        }

        if (target.getType().equals("TestPurpose")
                && !source.getType().equals("Comment")) {
            return false;
        }
        return true;
    }
}