/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.ui.cnf;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.navigator.ILinkHelper;

import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.RequalityPlugin;
import com.unitesk.requality.eclipse.core.TreesTracker;
import com.unitesk.requality.eclipse.editors.NodeEditorInput;
import com.unitesk.requality.eclipse.editors.browser.ReqMarker;
import com.unitesk.requality.eclipse.ui.editors.JSEditor;
import com.unitesk.requality.nodetypes.Document;
import com.unitesk.requality.nodetypes.Location;
import com.unitesk.requality.nodetypes.Requirement;

public class CNFLinkHelper implements ILinkHelper {
    private boolean ignoreCallForReqMarker = false;

    @Override
    public IStructuredSelection findSelection(IEditorInput anInput) {
        if (anInput instanceof NodeEditorInput && !ignoreCallForReqMarker) {
            NodeEditorInput input = (NodeEditorInput) anInput;
            try {
                TreeNode node = input.getStorage().getNode();
                // UniEditor
                if (node instanceof Requirement) {
                    IEditorReference eref = null;
                    try {
                        eref = RequalityPlugin.findJSEditorForNode(node
                                .getQualifiedId(), TreesTracker
                                .getResourceByNode(node).getProject());
                    } catch (PartInitException e) {
                        e.printStackTrace();
                    }
                    if (eref != null) {
                        IEditorPart epart = eref.getEditor(true);
                        if (epart instanceof JSEditor) {
                            JSEditor jsEditor = (JSEditor) epart;
                            if (jsEditor.getCurrentSelection() != null) {
                                return new StructuredSelection(
                                        jsEditor.getCurrentSelection());
                            }
                        }
                    }
                } else
                // ReqMarker
                if (node instanceof Document) {
                    Document doc = (Document) node;
                    IEditorReference eref = null;
                    try {
                        eref = RequalityPlugin.findMarkEditorForDocument(doc
                                .getQualifiedId(), TreesTracker
                                .getResourceByNode(doc).getProject());
                    } catch (PartInitException e) {
                        e.printStackTrace();
                    }
                    if (eref != null) {
                        IEditorPart epart = eref.getEditor(true);
                        if (epart instanceof ReqMarker) {
                            ReqMarker reqMarker = (ReqMarker) epart;
                            Location loc = reqMarker.getHighlightLocation();
                            if(loc==null) {
                                return null;
                            }
                            Requirement[] reqs = loc.getRequirements();
                            if (loc != null && reqs.length > 0) {
                                if(Arrays.asList(reqs).contains(RequalityCNF.getSelectedNode())) {
                                    return null;
                                }
                                return new StructuredSelection(reqs[0]);
                            }
                        }
                    }
                }
                return new StructuredSelection(node);
            } catch (CoreException e) {
                e.printStackTrace();
            }
        }
        ignoreCallForReqMarker = false;
        return null;
    }

    @Override
    public void activateEditor(IWorkbenchPage aPage,
            IStructuredSelection selection) {

        if (!(selection.getFirstElement() instanceof TreeNode)) {
            return; // find editor only for tree nodes
        }

        TreeNode node = (TreeNode) selection.getFirstElement();

        try {
            IEditorPart activeEditor = PlatformUI.getWorkbench()
                    .getActiveWorkbenchWindow().getActivePage()
                    .getActiveEditor();
            IEditorReference[] editorRefs = findRelevantEditors(node);
            IEditorPart curEditor = null;
            boolean findActiveEditor = false; // do not change active editor, if
                                              // it's listen selection change
            for (IEditorReference eRef : editorRefs) {
                curEditor = eRef.getEditor(false);
                if (activeEditor.equals(curEditor)) {
                    findActiveEditor = true;
                }
                if (curEditor instanceof ReqMarker) {
                    ReqMarker marker = (ReqMarker) curEditor;
                    if (node.getType().equals("Location")) {
                        // to prevent jump to requirement node
                        ignoreCallForReqMarker = true;
                        marker.setTargetLocation((Location) node);
                    }
                    if (node.getType().equals("Requirement")) {
                        Requirement req = (Requirement) node;
                        List<String> qids = req.getLocationQIds();
                        for (String locQId : qids) {
                            if (locQId.startsWith(marker.getDocument()
                                    .getQualifiedId())) {
                                ignoreCallForReqMarker = true; //don't go to findSelection()!
                                marker.setTargetLocation(req.getLocation(locQId));
                                break;
                            }
                        }
                    }
                }
                if (curEditor instanceof JSEditor) {
                    ignoreCallForReqMarker = true; //don't go to findSelection()!
                    ((JSEditor) curEditor).setSelection(node);
                }
            }
            if (curEditor != null && !findActiveEditor) {
                PlatformUI.getWorkbench().getActiveWorkbenchWindow()
                        .getActivePage().bringToTop(curEditor);
            }
        } catch (CoreException e1) {
            e1.printStackTrace();
        }
    }

    private IEditorReference[] findRelevantEditors(TreeNode node)
            throws CoreException {
        List<IEditorReference> result = new ArrayList<IEditorReference>();
        IEditorReference[] eRefs = PlatformUI.getWorkbench()
                .getActiveWorkbenchWindow().getActivePage()
                .getEditorReferences();

        for (IEditorReference eRef : eRefs) {
            IEditorPart editor = eRef.getEditor(false);

            // ReqMarker
            if (editor instanceof ReqMarker) {
                ReqMarker marker = (ReqMarker) editor;

                if (node.getType().equals("Requirement")) {
                    Requirement req = (Requirement) node;
                    List<String> qids = req.getLocationQIds();
                    for (String locQId : qids) {
                        if (locQId.startsWith(marker.getDocument()
                                .getQualifiedId())) {
                            result.add(eRef);
                            break;
                        }
                    }
                }
                if (node.getType().equals("Location")
                        || node.getType().equals("Document")) {
                    // for document and locations
                    if (node.getQualifiedId().startsWith(
                            marker.getDocument().getQualifiedId())) {
                        result.add(eRef);
                    }

                }
            }

            // JSEditor
            if (editor instanceof JSEditor) {
                JSEditor jsEditor = (JSEditor) editor;

                if (node.getQualifiedId().startsWith(
                        ((NodeEditorInput) jsEditor.getEditorInput())
                                .getStorage().getNode().getQualifiedId())) {
                    result.add(eRef);
                }
            }
        }
        return result.toArray(new IEditorReference[0]);
    }
}
