/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.views.properties;

import java.util.Iterator;
import java.util.LinkedHashMap;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetWidgetFactory;

import com.unitesk.requality.core.Attribute;
import com.unitesk.requality.core.AttributeType;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.editors.panels.SelectRequirementPanel;
import com.unitesk.requality.nodetypes.ReportData;
import com.unitesk.requality.nodetypes.Requirement;
import com.unitesk.requality.report.Template;

public class ReportDataPropertySection extends TreeNodePropertiesTab {
    private static String LBL_ROOT = "Root requirement:";
    private static String LBL_TEMPLATE = "Template:";
    // Widgets
    private SelectRequirementPanel rootReqPanel;
    private Combo templates;
    private AttributesPanel attrPanel;

    private LinkedHashMap<String, Template> tpls = new LinkedHashMap<String, Template>();
    private String selectedTemplateId;
    private ReportData reportData;
    private TreeDB db;

    @Override
    protected void setTreeNode(TreeNode node) {
        if(!node.getType().equals("ReportData")) {
            return;
        }
        reportData = (ReportData) node;
        db = node.getTreeDB();
        Requirement req;
        if (reportData.getStartRequirementQId()  == null) {
            req = (Requirement) db.getNode(Requirement.getTypeRootQId());
        } else {
            req = (Requirement) db.getNode(reportData.getStartRequirementQId());
        }
        attrPanel.setNode(node);
        rootReqPanel.setRequirement(req);

        selectedTemplateId = (String) node.getAttributeValue("_template_id");
        if (selectedTemplateId == null) {
            selectedTemplateId = tpls.keySet().iterator().next();
        }
        Iterator<String> it = tpls.keySet().iterator();
        String curId = "";
        int i = 0;
        for (; i < tpls.size() && !selectedTemplateId.equals(curId); i++) {
            curId = it.next();
        }
        templates.select(i - 1);
    }

    @Override
    public void createControls(Composite parent,
            TabbedPropertySheetPage aTabbedPropertySheetPage) {
        super.createControls(parent, aTabbedPropertySheetPage);
        parent.setLayout(new GridLayout(2, false));
        parent.setLayoutData(new GridData(GridData.FILL_BOTH));

        TabbedPropertySheetWidgetFactory wf = aTabbedPropertySheetPage.getWidgetFactory();
        wf.createLabel(parent, LBL_ROOT);
        rootReqPanel = new SelectRequirementPanel(parent, SWT.NONE, db);
        rootReqPanel.setCanSelectRoot(true);
        rootReqPanel.addListener(SWT.Modify, new Listener() {
            @Override
            public void handleEvent(Event event) {
                Requirement req = rootReqPanel.getSelectedRequirement();
                node.getTreeDB().startTransaction("Update Report Data");
                node.putAttribute(new Attribute(AttributeType.STRING,
                        "_root_requiremnt_qid", req.getQualifiedId()));
                node.saveAttributes();
                node.getTreeDB().commit();
            }
        });
        wf.createLabel(parent,LBL_TEMPLATE);
        createSelectTemplatePanel(parent).addListener(SWT.Modify, new Listener() {
            @Override
            public void handleEvent(Event event) {
                Iterator<String> it = tpls.keySet().iterator();
                for (int i = 0; i < templates.getSelectionIndex() + 1; i++) {
                    selectedTemplateId = it.next();
                }
                node.getTreeDB().startTransaction("Update Report Data");
                node.putAttribute(new Attribute(AttributeType.STRING,
                                "_template_id", selectedTemplateId));
                node.saveAttributes();
                node.getTreeDB().commit();
            }
        });

        attrPanel = new AttributesPanel(parent, wf);
        GridData data = new GridData(GridData.FILL_BOTH);
        data.horizontalSpan = 2;
        attrPanel.getControl().setLayoutData(data);
    }

    public Template getTemplate() {
        return tpls.get(selectedTemplateId);
    }

    private Composite createSelectTemplatePanel(Composite parent) {
        final Composite panel = new Composite(parent, SWT.NONE);
        panel.setLayout(new GridLayout());
        templates = new Combo(panel, SWT.NONE | SWT.READ_ONLY);
        tpls = new LinkedHashMap<String, Template>(Activator.getDefault()
                .getReportTemplates());
        if (tpls.size() == 0) {
            return panel;
        }
        if (selectedTemplateId == null) {
            selectedTemplateId = tpls.keySet().iterator().next();
        }

        for (Template tpl : tpls.values()) {
            templates.add(tpl.getName());
        }
        Iterator<String> it = tpls.keySet().iterator();
        String curId = "";
        int i = 0;
        for (; i < tpls.size() && !selectedTemplateId.equals(curId); i++) {
            curId = it.next();
        }
        templates.select(i-1);
        templates.addSelectionListener(new SelectionListener() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                panel.notifyListeners(SWT.Modify, new Event());
            }
            @Override
            public void widgetDefaultSelected(SelectionEvent e) {
                widgetSelected(e);
            }
        });
        return panel;
    }
}
