/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.tools;

import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.Reader;
import java.io.StringWriter;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Queue;
import java.util.Stack;
import java.util.UUID;

import javax.xml.namespace.QName;
import javax.xml.stream.XMLEventFactory;
import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLEventWriter;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.XMLStreamConstants;
import javax.xml.stream.events.Attribute;
import javax.xml.stream.events.Characters;
import javax.xml.stream.events.EndElement;
import javax.xml.stream.events.StartElement;
import javax.xml.stream.events.XMLEvent;

import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.documents.DocumentImporter;
import com.unitesk.requality.nodetypes.Location;
import com.unitesk.requality.nodetypes.Requirement;

public class Converter {
    private static QName REQ_START = new QName("span");
    private static QName REQ_END = new QName("span");

    // Stack element values
    private static String TAG_REQ = "1";
    private static String TAG_OTHER = "0";

    private static XMLEventFactory ef = XMLEventFactory.newInstance();

    // Attribute values
    private Stack<String> stack = new Stack<String>();
    private Queue<XMLEvent> xmlQueue = new LinkedList<XMLEvent>();
    private Map<String, ArrayList<XMLEvent>> locToXml = new HashMap<String, ArrayList<XMLEvent>>();

    // Results
    private Map<String, String[]> reqToLocations = new HashMap<String, String[]>();

    private String curLocId = null;
    private boolean firstTag=true;

    public static void usage() {
        System.out
                .println("Usage: java -jar converter.jar <input file> <output file>");
    }

    public static void main(String[] args) {
        String file = args[0];
//        String file = "/home/yuriy/svn/requality/distr/error/baselib-pam-start.html";
        if (!convert(file, file + ".xhtml")) {
            System.out.println("ERROR! " + file);
            ;
        }
    }

    @Deprecated
    public static boolean convert(String input, String output) {
        File inFile = new File(input);
        if (inFile.exists() && inFile.isFile()) {
            try {
                File tempFile = File.createTempFile("cvrt", ".html");
                FileReader reader = new FileReader(inFile);
                FileWriter writer = new FileWriter(tempFile);
                DocumentImporter.cleanHtml(reader, writer);
                writer.flush();
                File outFile = new File(output);

                if (outFile.getParentFile() != null) {
                    outFile.getParentFile().mkdirs();
                }

                new Converter().convert(
                        new FileReader(tempFile), new FileWriter(outFile));

                tempFile.delete();
                return true;
            } catch (Exception e) {
                e.printStackTrace();
                return false;
            }
        }
        return false;
    }

    /**
     * @param fReader
     * @param out
     * @return Map<requirement id, location ids []>
     */
    public Map<String, String[]> convert(Reader fReader, Writer out) {
        try {
            reqToLocations.clear();
            locToXml.clear();
            XMLEventReader eReader = XMLInputFactory.newInstance()
                    .createXMLEventReader(fReader);

            XMLEventWriter xmlWriter = XMLOutputFactory.newInstance()
                    .createXMLEventWriter(out);

            while (eReader.hasNext()) {
                XMLEvent event = eReader.nextEvent();
                processEvent(event);
            }

            for (XMLEvent xml : xmlQueue) {
                xmlWriter.add(xml);
            }
            fReader.close();
            xmlWriter.flush();
            xmlWriter.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return reqToLocations;
    }

    public void store(Map<String, String[]> rl, String docId, TreeDB db, Requirement groupNode) {
        if (db.getNode(docId) == null
                || !db.getNode(docId).getType().equals("Document")) {
            throw new RuntimeException("Document: " + docId + " in not in base");
        }
        db.startTransaction("Converter");
        for (String reqId : rl.keySet()) {
            String fullReqId = groupNode.getQualifiedId()+reqId;
            Requirement req = (Requirement) touchNode(db, fullReqId);
            for (String locId : rl.get(reqId)) {
                String locQId = docId + "/" + locId;
                if (db.getNode(locQId) == null) {
                    Location loc = (Location) db.getNode(docId).createChild(locId, "Location");
                    loc.setDescription(this.getLocationDescription(locId));
                    loc.saveAttributes();
                }
                req.addLocation(locQId);
            }
            req.saveAttributes();
        }
        db.commit();
    }
    public void store(Map<String, String[]> rl, String docId, TreeDB db) {
        this.store(rl, docId, db, (Requirement) db.getNode(Requirement.getTypeRootQId()));
    }

    private static TreeNode touchNode(TreeDB db, String qId) {
        if (qId.equals("/")) {
            return db.getRootNode();
        }

        TreeNode node = db.getNode(qId);
        if (node == null && node != db.getRootNode()) {
            int ind = qId.lastIndexOf("/");
            String parId = ind > 1 ? qId.substring(0, ind) : "";
            String id = ind >= 0 ? qId.substring(ind + 1) : "";

            TreeNode parent = touchNode(db, parId);
            node = parent.createChild(id);
        }
        return node;
    }

    private void processEvent(XMLEvent event) {
        switch (event.getEventType()) {
        case XMLStreamConstants.START_ELEMENT:
            processStartElement(event.asStartElement());
            break;
        case XMLStreamConstants.CHARACTERS:
            processCharacters(event.asCharacters());
            break;
        case XMLStreamConstants.END_ELEMENT:
            processEndElement(event.asEndElement());
            break;
        default:
            // out.add( event );
            break;
        }
    }

    private void addStartElement(StartElement se, String type) {
        if(curLocId!=null) {
            putDescXml(se);
        }
        xmlQueue.add(se);
        stack.push(type);
    }

    private void putDescXml(XMLEvent se) {
        if(locToXml.get(curLocId)==null) {
            locToXml.put(curLocId, new ArrayList<XMLEvent>());
        }
        locToXml.get(curLocId).add(se);
    }

    private String startReq(StartElement xmlElem, String type) {
        StartElement newTag = cleanReqTag(xmlElem);
        xmlQueue.add(newTag);
        stack.push(TAG_REQ);
        curLocId = UUID.randomUUID().toString();
        putDescXml(newTag);
        return curLocId;
    }

    private static StartElement createStartReqTag(String locId) {
        Attribute a = ef.createAttribute("class", "requality_text id_" + locId);
        ArrayList<Attribute> attrs = new ArrayList<Attribute>();
        attrs.add(a);
        StartElement spanBegin = ef.createStartElement(REQ_START,
                attrs.iterator(), null);
        return spanBegin;
    }

    private static StartElement cleanReqTag(StartElement xmlElem) {
        ArrayList<Attribute> newAttrs = new ArrayList<Attribute>();
        @SuppressWarnings("unchecked")
        Iterator<Attribute> i = xmlElem.getAttributes();
        while (i.hasNext()) {
            Attribute at = i.next();
            if (at.getName().equals(new QName("id"))) {
                continue;
            }
            if (at.getName().equals(new QName("class"))) {
                continue;
            }
            newAttrs.add(at);
        }
        StartElement newTag = ef.createStartElement(xmlElem.getName(),
                newAttrs.iterator(), xmlElem.getNamespaces());
        return newTag;
    }

    private void processStartElement(StartElement xmlElem) {

        Attribute attr = xmlElem.getAttributeByName(new QName("class"));
        if (attr == null) {
            addStartElement(xmlElem, TAG_OTHER);
            return;
        }

        String attrValue = attr.getValue();
        if (attrValue.equals("req") || attrValue.equals("struct")
                || attrValue.equals("uncheckable")) {
            startReq(xmlElem, attrValue);

            // Store requirement to location map
            Attribute aReqIds = xmlElem.getAttributeByName(new QName("id"));
            if(aReqIds==null) {
                return;
            }
            String reqIds[] = aReqIds.getValue().split(";");
            for (String reqId : reqIds) {
                String r = "/" + reqId.replace('.', '/');
                if (reqToLocations.containsKey(r)) {
                    ArrayList<String> newlst = new ArrayList<String>(
                            Arrays.asList(reqToLocations.get(r)));
                    newlst.add(curLocId);
                    reqToLocations.put(r, newlst.toArray(new String[0]));
                } else {
                    reqToLocations.put(r, new String[] { curLocId });
                }
            }
            return;
        }

        addStartElement(xmlElem, TAG_OTHER);
    }

    private void processEndElement(EndElement xmlElem) {
        String top = stack.pop();
        if (top.equals(TAG_REQ)) {
            putDescXml(xmlElem);
            curLocId = null;
            firstTag=true;
        }
        xmlQueue.add(xmlElem);
        if(curLocId!=null) {
            putDescXml(xmlElem);
        }
    }

    private void processCharacters(Characters xmlText) {
        String text = xmlText.getData();
        if(text.trim().length()==0) return;
        text = text.replace("&lt;", "<");
        text = text.replace("&gt;", ">");
        text = text.replace("&amp;", "&");
        text = text.replace("&quot;", "\"");
        text = text.replace("&apos;", "'");
        XMLEvent chars = ef.createCharacters(text);
        putDescXml(chars);
        if (curLocId!=null) {
            xmlQueue.add(createStartReqTag(curLocId));
            if(firstTag) {
                firstTag=false;
                xmlQueue.add(createAStartElement(curLocId));
                xmlQueue.add(ef.createEndElement(new QName("a"), null));
            }
            xmlQueue.add(chars);
            xmlQueue.add(ef.createEndElement(REQ_END, null));
        } else {
            xmlQueue.add(chars);
        }
    }

    private static StartElement createAStartElement(String locId) {
        ArrayList<Attribute> attrs = new ArrayList<Attribute>();
        attrs.add(ef.createAttribute("id", "id_" + locId));
        attrs.add(ef.createAttribute("name", locId));
        attrs.add(ef.createAttribute("class", "requality_id"));
        return ef.createStartElement(new QName("a"), attrs.iterator(), null);
    }

    public String getLocationDescription(String locUUId) {
        try {
            StringWriter out = new StringWriter();
            XMLEventWriter xmlWriter;

            xmlWriter = XMLOutputFactory.newInstance().createXMLEventWriter(out);
            List<XMLEvent> events = locToXml.get(locUUId);
            for(XMLEvent e : events) {
                xmlWriter.add(e);
            }
            xmlWriter.flush();
            xmlWriter.close();
            return out.toString();
        } catch (Exception e) {
            e.printStackTrace();
        }
        
        return null;
    }
}
